@@ -278,7 +278,11 @@ void death_handler (int signal)
     struct tunnel *st, *st2;
     int sec;
     l2tp_log (LOG_CRIT, "%s: Fatal signal %d received\n", __FUNCTION__, signal);
+#ifdef USE_KERNEL
+        if (kernel_support || signal != SIGTERM) {
+#else
         if (signal != SIGTERM) {
+#endif
                 st = tunnels.head;
                 while (st)
                 {
@@ -349,7 +353,7 @@ int start_pppd (struct call *c, struct ppp_opts *opts)
     int flags;
 #endif
     int pos = 1;
-    int fd2;
+    int fd2 = -1;
 #ifdef DEBUG_PPPD
     int x;
 #endif
@@ -394,10 +398,10 @@ int start_pppd (struct call *c, struct ppp_opts *opts)
                 __FUNCTION__);
            return -EINVAL;
        }
+       memset(&sax, 0, sizeof(sax));
        sax.sa_family = AF_PPPOX;
        sax.sa_protocol = PX_PROTO_OL2TP;
-       sax.pppol2tp.pid = 0;
-       sax.pppol2tp.fd = server_socket;
+       sax.pppol2tp.fd = c->container->udp_fd;
        sax.pppol2tp.addr.sin_addr.s_addr = c->container->peer.sin_addr.s_addr;
        sax.pppol2tp.addr.sin_port = c->container->peer.sin_port;
        sax.pppol2tp.addr.sin_family = AF_INET;
@@ -408,6 +412,7 @@ int start_pppd (struct call *c, struct ppp_opts *opts)
        if (connect(fd2, (struct sockaddr *)&sax, sizeof(sax)) < 0) {
            l2tp_log (LOG_WARNING, "%s: Unable to connect PPPoL2TP socket.\n",
                 __FUNCTION__);
+           close(fd2);
            return -EINVAL;
        }
        stropt[pos++] = strdup ("plugin");
@@ -416,6 +421,17 @@ int start_pppd (struct call *c, struct ppp_opts *opts)
        stropt[pos] = (char *) malloc (10);
        snprintf (stropt[pos], 10, "%d", fd2);
         pos++;
+       if (c->container->lns) {
+        stropt[pos++] = strdup ("pppol2tp_lns_mode");
+        stropt[pos++] = strdup ("pppol2tp_tunnel_id");
+        stropt[pos] = (char *) malloc (10);
+        snprintf (stropt[pos], 10, "%d", c->container->ourtid);
+            pos++;
+        stropt[pos++] = strdup ("pppol2tp_session_id");
+        stropt[pos] = (char *) malloc (10);
+        snprintf (stropt[pos], 10, "%d", c->ourcid);
+            pos++;
+       }
         stropt[pos] = NULL;
     }
     else
@@ -484,7 +500,7 @@ int start_pppd (struct call *c, struct ppp_opts *opts)
         dup2 (fd2, 0);
         dup2 (fd2, 1);
 	close(fd2);
-
+       }
         /* close all the calls pty fds */
         st = tunnels.head;
         while (st)
@@ -492,12 +508,17 @@ int start_pppd (struct call *c, struct ppp_opts *opts)
             sc = st->call_head;
             while (sc)
             {
-                close (sc->fd);
+#ifdef USE_KERNEL
+                if (kernel_support) {
+                    close(st->udp_fd); /* tunnel UDP fd */
+                    close(st->pppox_fd); /* tunnel PPPoX fd */
+                } else
+#endif
+                    close (sc->fd); /* call pty fd */
                 sc = sc->next;
             }
             st = st->next;
         }
-       }
 
         /* close the UDP socket fd */
         close (server_socket);
@@ -615,6 +636,10 @@ void destroy_tunnel (struct tunnel *t)
        the memory pointed to by t->chal_us.vector at some other place */
     if (t->chal_them.vector)
         free (t->chal_them.vector);
+    if (t->pppox_fd > -1 )
+        close (t->pppox_fd);
+    if (t->udp_fd > -1 )
+        close (t->udp_fd);
     free (t);
     free (me);
 }
@@ -816,18 +841,12 @@ void lac_disconnect (int tid)
 
 struct tunnel *new_tunnel ()
 {
-    struct tunnel *tmp = malloc (sizeof (struct tunnel));
+    struct tunnel *tmp = calloc (1, sizeof (struct tunnel));
     unsigned char entropy_buf[2] = "\0";
     if (!tmp)
         return NULL;
-    tmp->control_seq_num = 0;
-    tmp->control_rec_seq_num = 0;
-    tmp->cLr = 0;
-    tmp->call_head = NULL;
-    tmp->next = NULL;
     tmp->debug = -1;
     tmp->tid = -1;
-    tmp->hello = NULL;
 #ifndef TESTING
 /*      while(get_call((tmp->ourtid = rand() & 0xFFFF),0,0,0)); */
 /*        tmp->ourtid = rand () & 0xFFFF; */
@@ -845,24 +864,16 @@ struct tunnel *new_tunnel ()
 #else
     tmp->ourtid = 0x6227;
 #endif
-    tmp->nego = 0;
-    tmp->count = 0;
-    tmp->state = 0;             /* Nothing */
     tmp->peer.sin_family = AF_INET;
-    tmp->peer.sin_port = 0;
     bzero (&(tmp->peer.sin_addr), sizeof (tmp->peer.sin_addr));
 #ifdef SANITY
     tmp->sanity = -1;
 #endif
     tmp->qtid = -1;
     tmp->ourfc = ASYNC_FRAMING | SYNC_FRAMING;
-    tmp->ourbc = 0;
     tmp->ourtb = (((_u64) rand ()) << 32) | ((_u64) rand ());
     tmp->fc = -1;               /* These really need to be specified by the peer */
     tmp->bc = -1;               /* And we want to know if they forgot */
-    tmp->hostname[0] = 0;
-    tmp->vendor[0] = 0;
-    tmp->secret[0] = 0;
     if (!(tmp->self = new_call (tmp)))
     {
         free (tmp);
@@ -872,21 +883,9 @@ struct tunnel *new_tunnel ()
     tmp->self->ourfbit = FBIT;
     tmp->rxspeed = DEFAULT_RX_BPS;
     tmp->txspeed = DEFAULT_TX_BPS;
-    tmp->lac = NULL;
-    tmp->lns = NULL;
-    tmp->chal_us.state = 0;
-    tmp->chal_us.secret[0] = 0;
     memset (tmp->chal_us.reply, 0, MD_SIG_SIZE);
-    tmp->chal_us.challenge = NULL;
-    tmp->chal_us.chal_len = 0;
-    tmp->chal_them.state = 0;
-    tmp->chal_them.secret[0] = 0;
     memset (tmp->chal_them.reply, 0, MD_SIG_SIZE);
-    tmp->chal_them.challenge = NULL;
-    tmp->chal_them.chal_len = 0;
     tmp->chal_them.vector = (unsigned char *) malloc (VECTOR_SIZE);
-    tmp->chal_us.vector = NULL;
-    tmp->hbit = 0;
     return tmp;
 }
 
